/* The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _GANDIW_FILE_VIEW_H

#define _GANDIW_FILE_VIEW_H	1

#include "gandiw_list_box.h"
#include <dirent.h>

typedef struct _GANDIW_file_list GANDIW_file_list;

struct _GANDIW_file_list {
	GANDIW_widget *		core;	/* common for all widgets */
	GANDIW_list_box *	lstwid;	/* list box */
	char *			pwd;	/* work directory */
	bool			hide;	/* do not show hidden files */
	bool			sort;	/* throw files at the end */
	chtype *		light;	/* highlighting */
};

#define GANDIW_file_list_light_FILE	0
#define GANDIW_file_list_light_DIR	1
#define GANDIW_file_list_light_LINK	2
#define GANDIW_file_list_light_FIFO	3
#define GANDIW_file_list_light_SOCK	4
#define GANDIW_file_list_light_DOOR	5
#define GANDIW_file_list_light_BLK	6
#define GANDIW_file_list_light_CHR	7
#define GANDIW_file_list_light_ORPHAN	8
#define GANDIW_file_list_light_UID	9
#define GANDIW_file_list_light_GID	10
#define GANDIW_file_list_light_SBOW	11
#define GANDIW_file_list_light_OW	12
#define GANDIW_file_list_light_SB	13
#define GANDIW_file_list_light_EXEC	14

extern GANDIW_file_list GANDIW_file_list_dfl;
extern GANDIW_widget GANDIW_file_list_dfl_core;

extern chtype GANDIW_file_list_light[16];

extern GANDIW_file_list *GANDIW_file_list_make(int nlines, int ncols, int by,		int bx, void *parent);
extern void GANDIW_file_list_light_reset(chtype *a);
extern void GANDIW_file_list_kill(GANDIW_file_list *w);
extern int GANDIW_file_list_resize(GANDIW_file_list *w, int nlines, int ncols,		int by, int bx);
extern bool GANDIW_file_list_act_sel(GANDIW_list_box *w, size_t i);
extern int GANDIW_file_list_filetype(char *fn);
extern int GANDIW_file_list_show(GANDIW_file_list *w);
extern int GANDIW_file_list_sort(GANDIW_file_list *w);

#define GANDIW_file_list_vtw(w)		( (GANDIW_file_list *)(w) )
#define GANDIW_file_list_lstwid(w)	( GANDIW_file_list_vtw(w)->lstwid )

#define GANDIW_file_list_get_pwd(w)	( GANDIW_file_list_vtw(w)->pwd )
#define GANDIW_file_list_get_hide(w)	( GANDIW_file_list_vtw(w)->hide )
#define GANDIW_file_list_get_sort(w)	( GANDIW_file_list_vtw(w)->sort )
#define GANDIW_file_list_get_light(w)	( GANDIW_file_list_vtw(w)->light )
#define GANDIW_file_list_get_Nlight(w,n)						( GANDIW_file_list_vtw(w)->light[n] )

extern char *GANDIW_file_list_set_pwd(GANDIW_file_list *w, char *val);
extern bool GANDIW_file_list_set_hide(GANDIW_file_list *w, bool val);
extern bool GANDIW_file_list_set_sort(GANDIW_file_list *w, bool val);
extern chtype GANDIW_file_list_set_Nlight(GANDIW_file_list *w,				unsigned short index, chtype val);
extern chtype *GANDIW_file_list_set_light(GANDIW_file_list *w, chtype *val);

#endif /* gandiw_file_list.h */

