/* The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _GANDIW_LIST_BOX_H

#define _GANDIW_LIST_BOX_H	1

#include "gandiw_scroll_box.h"

typedef struct _GANDIW_list_box GANDIW_list_box;
typedef struct _GANDIW_list_box_item GANDIW_list_box_item;

typedef bool(* GANDIW_list_box_act_sel)(GANDIW_list_box *, size_t);

struct _GANDIW_list_box {
	GANDIW_widget *		core;		/* common for all widgets */
	attr_t			win_attr;
	short			win_pair;
	attr_t			cursor_attr;
	short			cursor_pair;
	attr_t			sel_attr;
	short			sel_pair;
	GANDIW_scroll_box	*scr;
	GANDIW_list_box_item **	item;
	bool			allow_show;
	size_t			cursor;
	bool			allow_sel;
	size_t			maxsel;
	bool			sort;
	struct {
	GANDIW_list_box_act_sel	sel;
	} act;
};

extern const int GANDIW_list_box_item_ftype_none;
extern const int GANDIW_list_box_item_ftype_free;
extern const int GANDIW_list_box_item_ftype_lfree;
extern const int GANDIW_list_box_item_ftype_cust;

struct _GANDIW_list_box_item {
	chtype *	caption;
	bool		selected;
	void *		userptr;
	int		ftype;
};

extern GANDIW_widget GANDIW_list_box_dfl_core;
extern GANDIW_list_box GANDIW_list_box_dfl;

extern GANDIW_list_box *GANDIW_list_box_make(int nlines, int ncols, int by,		int bx, void *parent);
extern int GANDIW_list_box_resize(GANDIW_list_box *w, int nlines, int ncols,		int by, int bx);
extern void GANDIW_list_box_kill(GANDIW_list_box *w);
extern size_t GANDIW_list_box_add_Nitem(GANDIW_list_box *w,				const chtype *item);
#define GANDIW_list_box_add_item(w,i)							( GANDIW_list_box_add_Nitem((w), (i)) == GANDIW_list_box_count(w) ?			ERR : OK )
extern void GANDIW_list_box_show(GANDIW_list_box *w);
extern void GANDIW_list_box_scr_cursor(GANDIW_list_box *w);
extern void GANDIW_list_box_mv_cursor(GANDIW_list_box *w, bool down);
extern void GANDIW_list_box_remptr(void **ptr, int rule);
extern int GANDIW_list_box_rem_Nitem(GANDIW_list_box *w, long long index);
extern size_t GANDIW_list_box_longest(GANDIW_list_box *w);
extern size_t GANDIW_list_box_count_sel(GANDIW_list_box *w);
extern void GANDIW_list_box_refresh(GANDIW_list_box *w);
extern void GANDIW_list_box_clear_sel(GANDIW_list_box *w);
extern GANDIW_list_box_item **GANDIW_list_box_clear(GANDIW_list_box *w);
extern int GANDIW_list_box_driver(GANDIW_list_box *w, int k);

#define GANDIW_list_box_vtw(w)	((GANDIW_list_box *)(w))

#define GANDIW_list_box_window(w) GANDI_get_Nwindow((w), 0)

#define GANDIW_list_box_scr(w)	( GANDIW_list_box_vtw(w)->scr )
#define GANDIW_list_box_vscr(w)								( GANDIW_scroll_box_vscr(GANDIW_list_box_vtw(w)->scr) )
#define GANDIW_list_box_hscr(w)								( GANDIW_scroll_box_hscr(GANDIW_list_box_vtw(w)->scr) )

#define GANDIW_list_box_freeze(w,val)	{						GANDIW_scroll_bar_set_allow_show(GANDIW_list_box_vscr(w), !(val));		GANDIW_scroll_bar_set_allow_show(GANDIW_list_box_hscr(w), !(val));		GANDIW_list_box_set_allow_show((w), !(val)); }

#define GANDIW_list_box_fit_scr(w) {							GANDIW_scroll_box_set_rows((w)->scr, GANDIW_list_box_count(w));			GANDIW_scroll_box_set_cols((w)->scr, GANDIW_list_box_longest(w)); }

#define GANDIW_list_box_count(w)							GANDIW_scroll_box_get_rows( GANDIW_list_box_scr(w) )

/*	whether index is an existing item index (FALSE - yes, TRUE - no) */
#define GANDIW_list_box_check(w, index)							( ( ((index) >= 0) && ((index) < GANDIW_list_box_count(w)) ) ?			FALSE : TRUE )

#define GANDIW_list_box_get_cursor(w)							( GANDIW_list_box_vtw(w)->cursor )
extern size_t GANDIW_list_box_set_cursor(GANDIW_list_box *w, size_t val);

#define GANDIW_list_box_get_Nitem(w,index)						( GANDIW_list_box_check((w), GANDI_fitu(index)) ? NULL :				GANDIW_list_box_vtw(w)->item[GANDI_fitu(index)] )
#define GANDIW_list_box_set_Nitem(w,index,val)						( GANDIW_list_box_check((w), GANDI_fitu(index)) ? NULL :				( GANDIW_list_box_vtw(w)->item[GANDI_fitu(index)] = (val) ) )

#define GANDIW_list_box_get_Nsel(w,index)						( GANDIW_list_box_check((w), GANDI_fitu(index)) ? FALSE :				( GANDIW_list_box_get_Nitem( (w),							GANDI_fitu(index) )->selected ) )
extern bool GANDIW_list_box_set_Nsel(GANDIW_list_box *w, long long index,		bool value, bool check);

#define GANDIW_list_box_get_Nuserptr(w,index)						( GANDIW_list_box_check((w), GANDI_fitu(index)) ? NULL :				GANDIW_list_box_get_Nitem((w), GANDI_fitu(index))->userptr )
#define GANDIW_list_box_set_Nuserptr(w,index,ptr)					( GANDIW_list_box_check((w), GANDI_fitu(index)) ? NULL :				( GANDIW_list_box_get_Nitem( (w),							GANDI_fitu(index))->userptr = (ptr) ) )

extern chtype *GANDIW_list_box_set_Ncap(GANDIW_list_box *w, long long index,		const chtype *s);
#define GANDIW_list_box_get_Ncap(w,index)						( GANDIW_list_box_check((w), GANDI_fitu(index)) ? NULL :				GANDIW_list_box_get_Nitem((w), GANDI_fitu(index))->caption )

#define GANDIW_list_box_get_Nftype(w,index)						( GANDIW_list_box_check((w), GANDI_fitu(index)) ?					GANDIW_list_box_item_ftype_none :						GANDIW_list_box_get_Nitem((w), GANDI_fitu(index))->ftype )
#define GANDIW_list_box_set_Nftype(w,index,val)						( GANDIW_list_box_check((w), GANDI_fitu(index)) ?					GANDIW_list_box_item_ftype_none :						( GANDIW_list_box_get_Nitem( (w),							GANDI_fitu(index))->ftype = (val) ) )

#define GANDIW_list_box_get_allow_sel(w)						( GANDIW_list_box_vtw(w)->allow_sel )
#define GANDIW_list_box_set_allow_sel(w,value)						( GANDIW_list_box_vtw(w)->allow_sel = (value) )

#define GANDIW_list_box_get_maxsel(w)	( GANDIW_list_box_vtw(w)->maxsel )
#define GANDIW_list_box_set_maxsel(w,value)						( GANDIW_list_box_vtw(w)->maxsel = (value) )

#define GANDIW_list_box_get_win_attr(w) ( GANDIW_list_box_vtw(w)->win_attr )
extern attr_t GANDIW_list_box_set_win_attr(GANDIW_list_box *w, attr_t val);

#define GANDIW_list_box_get_win_pair(w) ( GANDIW_list_box_vtw(w)->win_pair )
extern short GANDIW_list_box_set_win_pair(GANDIW_list_box *w, short val);

#define GANDIW_list_box_get_cursor_attr(w)						( GANDIW_list_box_vtw(w)->cursor_attr )
extern attr_t GANDIW_list_box_set_cursor_attr(GANDIW_list_box *w, attr_t val);

#define GANDIW_list_box_get_cursor_pair(w)						( GANDIW_list_box_vtw(w)->cursor_pair )
extern short GANDIW_list_box_set_cursor_pair(GANDIW_list_box *w, short val);

#define GANDIW_list_box_get_sel_attr(w) ( GANDIW_list_box_vtw(w)->sel_attr )
extern attr_t GANDIW_list_box_set_sel_attr(GANDIW_list_box *w, attr_t val);

#define GANDIW_list_box_get_sel_pair(w) ( GANDIW_list_box_vtw(w)->sel_pair )
extern short GANDIW_list_box_set_sel_pair(GANDIW_list_box *w, short val);

#define GANDIW_list_box_get_allow_show(w) (GANDIW_list_box_vtw(w)->allow_show)
extern bool GANDIW_list_box_set_allow_show(GANDIW_list_box *w, bool val);

#define GANDIW_list_box_get_sort(w) ( GANDIW_list_box_vtw(w)->sort )
extern bool GANDIW_list_box_set_sort(GANDIW_list_box *w, bool val);

#define GANDIW_list_box_setact_sel(w,val)						(GANDIW_list_box_vtw(w)->act.sel = val)
#define GANDIW_list_box_getact_sel(w)	(GANDIW_list_box_vtw(w)->act.sel)

#define GANDIW_list_box_2text(w,cr)							GANDI_ch_2text_cr((cr), 1,								GANDIW_list_box_get_win_attr(								(w) ? (w) : &GANDIW_list_box_dfl ) |					COLOR_PAIR( GANDIW_list_box_get_win_pair(						(w) ? (w) : &GANDIW_list_box_dfl ) )				)

#endif /* gandiw_list_box.h */

