/* The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _GANDIW_SCROLL_BAR_H

#define _GANDIW_SCROLL_BAR_H	1

#include "gandi_core.h"

typedef struct _GANDIW_scroll_bar GANDIW_scroll_bar;

typedef void (* GANDIW_scroll_bar_act_scr)(void *);

struct _GANDIW_scroll_bar {
	GANDIW_widget		*core;		/* common for all widgets */
	attr_t			win_attr;
	short			win_pair;
	bool			allow_show;
	int			alignment;	/* vertical or horizontal */
	size_t			maximum;	/* maximum value */
	size_t			value;		/* value in store */
	void			*owner;		/* widget to be sent to
						   act.scr */
	struct {
	GANDIW_scroll_bar_act_scr	scr;	/* function to be called on
						   scr */
	} act;
};

extern GANDIW_widget GANDIW_scroll_bar_dfl_core;
extern GANDIW_scroll_bar GANDIW_scroll_bar_dfl;

extern const short GANDIW_scroll_bar_horizontal;
extern const short GANDIW_scroll_bar_vertical;

extern GANDIW_scroll_bar *GANDIW_scroll_bar_make(int nlines, int ncols, int by,		int bx, void *parent, int alignment, int maximum);
extern int GANDIW_scroll_bar_resize(GANDIW_scroll_bar *w, int nlines, int ncols,	int by, int bx);
extern int GANDIW_scroll_bar_show(GANDIW_scroll_bar *w);
extern long long GANDIW_scroll_bar_scr_val(GANDIW_scroll_bar *w, long long n);
extern size_t GANDIW_scroll_bar_scr(GANDIW_scroll_bar *w, bool down);
extern size_t GANDIW_scroll_bar_scr_edge(GANDIW_scroll_bar *w, bool end);
extern int GANDIW_scroll_bar_driver(GANDIW_scroll_bar *w, int k);

#define GANDIW_scroll_bar_vtw(w)	( (GANDIW_scroll_bar *)w )

#define GANDIW_scroll_bar_callact(w) { \
	if (GANDIW_scroll_bar_getact_scr(w)) GANDIW_scroll_bar_getact_scr(w) (\
		(GANDIW_scroll_bar_get_owner(w)) ? \
			GANDIW_scroll_bar_get_owner(w) : (w) ); }

#define GANDIW_scroll_bar_get_alignment(w)						( GANDIW_scroll_bar_vtw(w)->alignment )

#define GANDIW_scroll_bar_get_value(w)							( (GANDIW_scroll_bar_vtw(w))->value )
extern size_t GANDIW_scroll_bar_set_value(GANDIW_scroll_bar *w,				long long value);

#define GANDIW_scroll_bar_get_maximum(w)						( (GANDIW_scroll_bar_vtw(w))->maximum )
extern size_t GANDIW_scroll_bar_set_maximum(GANDIW_scroll_bar *w,				long long value);

#define GANDIW_scroll_bar_getact_scr(w)							( GANDIW_scroll_bar_vtw(w)->act.scr )
#define GANDIW_scroll_bar_setact_scr(w,val)						( GANDIW_scroll_bar_vtw(w)->act.scr = (val) )

#define GANDIW_scroll_bar_get_owner(w)							( GANDIW_scroll_bar_vtw(w)->owner )
#define GANDIW_scroll_bar_set_owner(w,val)						( GANDIW_scroll_bar_vtw(w)->owner = (val) )

#define GANDIW_scroll_bar_set_max	GANDIW_scroll_bar_set_maximum
#define GANDIW_scroll_bar_set_val	GANDIW_scroll_bar_set_value
#define GANDIW_scroll_bar_get_max	GANDIW_scroll_bar_get_maximum
#define GANDIW_scroll_bar_get_val	GANDIW_scroll_bar_get_value

#define GANDIW_scroll_bar_get_allow_show(w)						( GANDIW_scroll_bar_vtw(w)->allow_show )
extern bool GANDIW_scroll_bar_set_allow_show(GANDIW_scroll_bar *w, bool val);

#endif /* gandiw_scroll_bar.h */

