/* The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _GANDIW_TEXT_EDIT_H

#define _GANDIW_TEXT_EDIT_H		1

#include "gandiw_scroll_box.h"

typedef struct _GANDIW_text_edit GANDIW_text_edit;

struct _GANDIW_text_edit {
	GANDIW_widget *			core;
	attr_t				text_attr;
	short				text_pair;
	GANDIW_scroll_box *		scr;
	bool				allow_show;
	chtype **			text;
	size_t				currow;
	size_t				curcol;
	bool				viewmode;
	bool				editable;
	size_t				maxrow;
	size_t				maxcol;
	bool				overwrite;
	unsigned char			timer;
};

extern GANDIW_widget GANDIW_text_edit_dfl_core;
extern GANDIW_text_edit GANDIW_text_edit_dfl;

extern GANDIW_text_edit *GANDIW_text_edit_make(int nlines, int ncols, int by,		int bx, void *parent);
extern void GANDIW_text_edit_attach(GANDIW_text_edit *w);
extern void GANDIW_text_edit_dettach(GANDIW_text_edit *w);
extern void GANDIW_text_edit_kill(GANDIW_text_edit *w);
extern int GANDIW_text_edit_resize(GANDIW_text_edit *w, int nlines, int ncols,		int by, int bx);
extern int GANDIW_text_edit_add_ch(GANDIW_text_edit *w, const chtype c);
extern bool GANDIW_text_edit_timer(GANDIW_text_edit *w);
extern bool GANDIW_text_edit_show_cursor(GANDIW_text_edit *w);
extern void GANDIW_text_edit_show(GANDIW_text_edit *w);
extern int GANDIW_text_edit_mv_hcur(GANDIW_text_edit *w, bool right);
extern int GANDIW_text_edit_mv_vcur(GANDIW_text_edit *w, bool down);
extern void GANDIW_text_edit_mv_curyx(GANDIW_text_edit *w, size_t y,			size_t x);
extern int GANDIW_text_edit_rm_ch(GANDIW_text_edit *w, bool cur);
extern bool GANDIW_text_edit_mv_edge(GANDIW_text_edit *w, bool end);
extern chtype **GANDIW_text_edit_cr2wnt(GANDIW_text_edit *w,				const char *text);
extern GANDIT_drvret GANDIW_text_edit_driver(GANDIW_text_edit *w, int k);

#define GANDIW_text_edit_vtw(w)		( (GANDIW_text_edit *)w )
#define GANDIW_text_edit_scr(w)		( GANDIW_text_edit_vtw(w)->scr )
#define GANDIW_text_edit_vscr(w)							( GANDIW_scroll_box_vscr(GANDIW_text_edit_scr(w)) )
#define GANDIW_text_edit_hscr(w)							( GANDIW_scroll_box_hscr(GANDIW_text_edit_scr(w)) )

#define GANDIW_text_edit_freeze(w,val)	{						GANDIW_scroll_bar_set_allow_show(GANDIW_text_edit_vscr(w), !(val));		GANDIW_scroll_bar_set_allow_show(GANDIW_text_edit_hscr(w), !(val));		GANDIW_text_edit_set_allow_show((w), !(val)); }

#define GANDIW_text_edit_get_text(w)	( GANDIW_text_edit_vtw(w)->text )
extern chtype **GANDIW_text_edit_set_text(GANDIW_text_edit *w,				const chtype **val);

#define GANDIW_text_edit_get_currow(w)	( GANDIW_text_edit_vtw(w)->currow )
extern size_t GANDIW_text_edit_set_currow(GANDIW_text_edit *w, size_t val);

#define GANDIW_text_edit_get_curcol(w)	( GANDIW_text_edit_vtw(w)->curcol )
extern size_t GANDIW_text_edit_set_curcol(GANDIW_text_edit *w, size_t val);

#define GANDIW_text_edit_get_viewmode(w)						( GANDIW_text_edit_vtw(w)->viewmode )
extern bool GANDIW_text_edit_set_viewmode(GANDIW_text_edit *w, bool val);

#define GANDIW_text_edit_get_editable(w) ( GANDIW_text_edit_vtw(w)->editable )
#define GANDIW_text_edit_set_editable(w,val)						( GANDIW_text_edit_vtw(w)->editable = val )

#define GANDIW_text_edit_get_maxrow(w)	( GANDIW_text_edit_vtw(w)->maxrow )
extern size_t GANDIW_text_edit_set_maxrow(GANDIW_text_edit *w, size_t val);

#define GANDIW_text_edit_get_maxcol(w)	( GANDIW_text_edit_vtw(w)->maxcol )
extern size_t GANDIW_text_edit_set_maxcol(GANDIW_text_edit *w, size_t val);

#define GANDIW_text_edit_get_overwrite(w)						( GANDIW_text_edit_vtw(w)->overwrite )
extern bool GANDIW_text_edit_set_overwrite(GANDIW_text_edit *w, bool val);

#define GANDIW_text_edit_get_allow_show(w)						( GANDIW_text_edit_vtw(w)->allow_show )
extern bool GANDIW_text_edit_set_allow_show(GANDIW_text_edit *w, bool val);

#define GANDIW_text_edit_get_text_attr(w)						( GANDIW_text_edit_vtw(w)->text_attr )
extern attr_t GANDIW_text_edit_set_text_attr(GANDIW_text_edit *w, attr_t val);

#define GANDIW_text_edit_get_text_pair(w)						( GANDIW_text_edit_vtw(w)->text_pair )
extern short GANDIW_text_edit_set_text_pair(GANDIW_text_edit *w, short val);

#define GANDIW_text_edit_wnt2cr(w)							GANDI_ch_2cr((const chtype **)GANDIW_text_edit_get_text(w))

/*	If cursor is visible */
#define GANDIW_text_edit_curvis(w)							( GANDIW_text_edit_vtw(w)->currow >= GANDIW_scroll_bar_get_val(				GANDIW_text_edit_vscr(GANDIW_text_edit_vtw(w)) ) &&			GANDIW_text_edit_vtw(w)->curcol >= GANDIW_scroll_bar_get_val(				GANDIW_text_edit_hscr(GANDIW_text_edit_vtw(w)) ) )

#define GANDIW_text_edit_yxchar(w,y,x)							( ( GANDIW_text_edit_get_text(w)[y] ) ?						( ( GANDIW_text_edit_get_text(w)[y][x] ) ?					(GANDIW_text_edit_get_text(w)[y][x]) :						(' ' | GANDIW_text_edit_get_text_attr(w) |						COLOR_PAIR(GANDIW_text_edit_get_text_pair(w)) ) ) :			(' ' | GANDIW_text_edit_get_text_attr(w) |						COLOR_PAIR(GANDIW_text_edit_get_text_pair(w) ) ) )

#endif /* gandiw_text_edit.h */

